'-----------------------------------------------------------------------------------------
'Burkhard Kainka
'morse.bas
'morce code decoder with Mega169 and LCD butterfly driver
'using butterfly joystick as morse key
'-----------------------------------------------------------------------------------------

$regfile = "m169def.dat"                                    ' specify the used micro
$crystal = 8000000                                          ' used crystal frequency
$baud = 19200                                               ' use baud rate
$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 10                                               ' default use 10 for the SW stack
$framesize = 40                                             ' default use 40 for the frame space

$lib "lcd_butterfly.lbx"

'need to dim this byte
Dim _butterfly_digit As Byte

Dim Text As String * 10
Dim Key As Byte
Dim C As Byte
Dim Code(255) As Byte
Dim N As Integer
Dim Timepressed As Word
Dim Timenotpressed As Word
Dim Charactercode As Byte

Declare Sub Getkey
Declare Sub Getkeytimes
Declare Sub Gecharactercode
Declare Sub Initcode

Config Portb = 32
Portb = &HF0                                                'pullups for joystick

Config Timer1 = Pwm , Prescale = 8 , Pwm = 10 , Compare A Pwm = Clear Down
Pwm1a = 0                                                   'no sound


Initcode                                                    'prepare code table
Cls
Text = "MORSE "
Lcd Text

Do
  Gecharactercode
  C = Code(charactercode)
  Text = Text + Chr(c)
  Text = Right(text , 6)
  Cls
  Lcd Text
Loop



Sub Getkey
  Key = &HFF - Pinb
  Key = Key And &HD0
  If Key > 0 Then
    Pwm1a = 80
  Else
    Pwm1a = 0
  End If
End Sub

Sub Getkeytimes
  Timepressed = 0
  Timenotpressed = 0
  Do
    Getkey
  Loop Until Key > 0
  Do
    Waitms 1
    Timepressed = Timepressed + 1
    Getkey
  Loop Until Key = 0
  Do
    Waitms 1
    Timenotpressed = Timenotpressed + 1
    Getkey
  Loop Until Key > 0 Or Timenotpressed > 200
End Sub

Sub Gecharactercode
  Charactercode = 1
  Do
    Getkeytimes
    Charactercode = Charactercode * 2
    If Timepressed > 150 Then
      Charactercode = Charactercode + 1
    End If
  Loop Until Timenotpressed > 200
End Sub

Sub Initcode
  Code(1) = 32
  Code(2) = 69                                              ' "E",      . , 00000010, ASCII 69
  Code(3) = 84                                              ' "T",      - , 00000011, ASCII 84
  Code(4) = 73                                              ' "I",     .. , 00000100, ASCII 73
  Code(5) = 65                                              ' "A",     .- , 00000110, ASCII 65
  Code(6) = 78                                              ' "N",     -. , 00000110, ASCII 78
  Code(7) = 77                                              ' "M",     -- , 00000111, ASCII 77
  Code(8) = 83                                              ' "S",    ... , 00001000, ASCII 83
  Code(9) = 85                                              ' "U",    ..- , 00001001, ASCII 85
  Code(10) = 82                                             ' "R",    .-. , 00001010, ASCII 82
  Code(11) = 87                                             ' "W",    .-- , 00001011, ASCII 87
  Code(12) = 68                                             ' "D",    -.. , 00001100, ASCII 68
  Code(13) = 75                                             ' "K",    -.- , 00001101, ASCII 75
  Code(14) = 71                                             ' "G",    --. , 00001110, ASCII 71
  Code(15) = 79                                             ' "O",    --- , 00001111, ASCII 79
  Code(16) = 72                                             ' "H",   .... , 00010000, ASCII 72
  Code(17) = 86                                             ' "V",   ...- , 00010001, ASCII 86
  Code(18) = 70                                             ' "F",   ..-. , 00010010, ASCII 70
  Code(19) = 32
  Code(20) = 76                                             ' "L",   .-.. , 00010100, ASCII 76
  Code(21) = 32
  Code(22) = 80                                             ' "P",   .--. , 00010110, ASCII 80
  Code(23) = 74                                             ' "J",   .--- , 00010111, ASCII 74
  Code(24) = 66                                             ' "B",   -... , 00011000, ASCII 66
  Code(25) = 88                                             ' "X",   -..- , 00011001, ASCII 88
  Code(26) = 67                                             ' "C",   -.-. , 00011010, ASCII 67
  Code(27) = 89                                             ' "Y",   -.-- , 00011011, ASCII 89
  Code(28) = 90                                             ' "Z",   --.. , 00011100, ASCII 90
  Code(29) = 81                                             ' "Q",   --.- , 00011101, ASCII 81
  Code(30) = 32
  Code(31) = 32
  Code(32) = 53                                             ' "5",  ..... , 00100000, ASCII 53
  Code(33) = 52                                             ' "4",  ....- , 00100001, ASCII 52
  Code(34) = 32
  Code(35) = 51                                             ' "3",  ...-- , 00100011, ASCII 51
  Code(36) = 32
  Code(37) = 32
  Code(38) = 32
  Code(39) = 50                                             ' "2",  ..--- , 00100111, ASCII 50
  Code(40) = 32
  Code(41) = 32
  Code(42) = 32
  Code(43) = 32
  Code(44) = 32
  Code(45) = 32
  Code(46) = 32
  Code(47) = 49                                             ' "1",  .---- , 00101111, ASCII 49
  Code(48) = 54                                             ' "6",  -.... , 00110000, ASCII 54
  Code(49) = 32
  Code(50) = 32
  Code(51) = 32
  Code(52) = 32
  Code(53) = 32
  Code(54) = 32
  Code(55) = 32
  Code(56) = 55                                             ' "7",  --... , 00111000, ASCII 55
  Code(57) = 32
  Code(58) = 32
  Code(59) = 32
  Code(60) = 56                                             ' "8",  ---.. , 00111100, ASCII 56
  Code(61) = 32
  Code(62) = 57                                             ' "9",  ----. , 00111110, ASCII 57
  Code(63) = 48                                             ' "0",  ----- , 00111111, ASCII 48
  For N = 64 To 255
    Code(n) = 32
  Next N
End Sub


Lcd_butterfly_data:
Data 0%                                                     ' space
Data 0%                                                     ' !
Data 0%                                                     '""
Data 0%                                                     ' #
Data 0%                                                     '$
Data 0%                                                     '  %
Data 0%                                                     ' &
Data 0%                                                     '  '
Data 0%                                                     '  (
Data 0%                                                     '  )
Data 0%                                                     ' *
Data 0%                                                     ' +
Data 0%                                                     ' ,
Data 0%                                                     ' -
Data 0%                                                     ' .
Data 0%                                                     '/
Data &H5559%                                                ' 0
Data &H0118%                                                '1
Data &H1E11%                                                ' 2
Data &H1B11%                                                ' 3
Data &H0B50%                                                ' 4
Data &H1B41%                                                ' 5
Data &H1F41%                                                '6
Data &H0111%                                                ' 7
Data &H1F51%                                                ' 8
Data &H1B51%                                                '9
Data 0%                                                     ':
Data 0%                                                     ';
Data 0%                                                     '<
Data 0%                                                     '=
Data 0%                                                     '>
Data 0%                                                     '?
Data 0%                                                     '@
Data &H0F51%                                                ' A
Data &H3991% , &H1441% , &H3191% , &H1E41% , &H0E41% , &H1D41% , &H0F50% , &H2080% , &H1510% , &H8648% , &H1440% , &H0578%
Data &H8570% , &H1551% , &H0E51% , &H9551% , &H8E51% , &H9021% , &H2081% , &H1550% , &H4448% , &HC550% , &HC028% , &H2028% , &H5009%